#!/bin/sh

########################################################################
# Copyright (C) 2007-2009 Daniel Balsiger <mail<at>danielbalsiger.ch>  #
#                                                                      #
# This project evolved while my information science project and my     #
# master thesis that followed on.                                      #
#                                                                      #
# Check for latest versions at:                                        #
# https://subversion.cnds.unibe.ch/svn/adam/trunk/image-builder        #
#                                                                      #
# This program is free software; you can redistribute it and/or modify #
# it under the terms of the GNU General Public License version 2 as    #
# published by the Free Software Foundation.                           #
#                                                                      #
########################################################################

get_settings4()
{
    unset ipaddr ipmask ipnet ipdomain
    ipaddr=`get_var ${device} IP`
    ipmask=`get_var ${device} MASK`
    ipdomain=`get_var ${device} DOMAIN`
    if [ -n "${ipaddr}" ] && [ -n "${ipmask}" ] ; then
        ipnet=`ipcalc -n ${ipaddr}/${ipmask} | sed -e 's|NETWORK=||'`/${ipmask}
    else
        ipnet=""
    fi
}

get_settings6()
{
    unset ip6addr ip6mask ip6net ipd6domain
    ip6addr=`get_var ${device} IP6`
    ip6mask=`get_var ${device} MASK6`
    ip6domain=`get_var ${device} DOMAIN6`
    if [ -n "${ip6addr}" ] && [ -n "${ip6mask}" ] ; then
        #TODO calculate ip6net from "${ip6addr}" and  "${ip6mask}"
        # only prefixes % 16 = 0 are accepted at the moment
        let doubledots=ip6mask/16
        tmpaddr=$ip6addr 
        prefix=""
        while [ $doubledots -gt 0 ] ; do
            nibble=${tmpaddr%%:*}:
            tmpaddr=`echo $tmpaddr | sed -e 's|'$nibble'||'`
            prefix=$prefix$nibble
            let doubledots--
        done
        ip6net="$prefix/${ip6mask}"
    else
        ip6net=""
    fi
}

get_tmp_devices()
{
    if [ "${host}" == "${thishost}" ] ; then
        . /etc/conf.d/network.conf
    else
        . ${networkdir}/${host}.conf
    fi
    localdevices=0
    remotedevices=0
    for device in ${GLOBAL_DEVICES} ; do
        get_settings4
        get_settings6
        if [ "${host}" == "${thishost}" ] ; then
            let localdevices++
            eval localipnets$localdevices=${ipnet}
            eval localipaddrs$localdevices=${ipaddr}
            eval localipdomains$localdevices=${ipdomain}
            eval localip6nets$localdevices=${ip6net}
            eval localip6addrs$localdevices=${ip6addr}
            eval localip6domains$localdevices=${ip6domain}
        else
            let remotedevices++
            eval remoteipnets$remotedevices=${ipnet}
            eval remoteipaddrs$remotedevices=${ipaddr}
            eval remoteipdomains$remotedevices=${ipdomain}
            eval remoteip6nets$remotedevices=${ip6net}
            eval remoteip6addrs$remotedevices=${ip6addr}
            eval remoteip6domains$remotedevices=${ip6domain}
        fi
    done
}

analyse_staticnets()
{
    unset localdevices localipnets localipaddrs localipdomains localip6nets localip6addrs localip6domains
 
    echo "" >> ${tmphostsfile}
    echo "# according settings in /etc/network.d these hosts can be resolved: " >> ${tmphostsfile}

    host=${thishost}
    get_tmp_devices

    # for all localdevices calculate ip networks
    for ld in `seq ${localdevices}` ; do
        eval localipnet="\$localipnets$ld"
        eval localipaddr="\$localipaddrs$ld"
        eval localipdomain="\$localipdomains$ld"
        if [ -n "$localipnet" ] && [ -n "$localipaddr" ] ; then
            if [ -n "$localipdomain" ] ; then
                 echo "${localipaddr} ${thishost}.${localipdomain} ${thishost}" >> ${tmphostsfile}
            else
                 echo "${localipaddr} ${thishost}" >> ${tmphostsfile}
            fi
        fi
        eval localip6net="\$localip6nets$ld"
        eval localip6addr="\$localip6addrs$ld"
        eval localip6domain="\$localip6domains$ld" 
        if [ -n "$localip6net" ] && [ -n "$localip6addr" ] ; then
            if [ -n "$localip6domain" ] ; then
                echo "${localip6addr} ${thishost}.${localip6domain} ${thishost}" >> ${tmphostsfile}
            else
                echo "${localipaddr} ${thishost}" >> ${tmphostsfile}
            fi
        fi
        if [ -n "$localip6net" ] || [ -n "$localipnet" ] ; then
            for remote in ${networkdir}/*.conf ; do
                host=${remote##*/}
                host=${host%.conf}
                if [ "$thishost" != "$host" ] ;    then
                    # calculate ip networks of all remotedevices and add entry if a match is found
                    unset remotedevices remoteipnets remoteipaddrs remoteipdomains remoteip6nets remoteip6addrs remoteip6domains
                    get_tmp_devices
                    for rd in `seq ${remotedevices}` ; do
                        eval remoteipnet="\$remoteipnets$rd"
                        eval remoteipaddr="\$remoteipaddrs$rd"
                        eval remoteipdomain="\$remoteipdomains$rd"
                        if [ -n "${remoteipnet}" ] && [ -n "${remoteipaddr}" ] && [ "${localipnet}" == "${remoteipnet}" ] ; then
                            if [ -n "${remoteipdomain}" ] && [ "${localipdomain}" == "${remoteipdomain}" ] ; then
                                echo "${remoteipaddr} ${host}.${remoteipdomain} ${host}" >> ${tmphostsfile}
                            else
                                echo "${remoteipaddr} ${host}" >> ${tmphostsfile}
                            fi
                        fi
                        eval remoteip6net="\$remoteip6nets$rd"
                        eval remoteip6addr="\$remoteip6addrs$rd"
                        eval remoteip6domain="\$remoteip6domains$rd"
                        if [ -n "${remoteip6net}" ] && [ -n "${remoteip6addr}" ] && [ "${localip6net}" == "${remoteip6net}" ] ; then
                            if [ -n "${remoteip6domain}" ] && [ "${localip6domain}" == "${remoteip6domain}" ] ; then
                                echo "${remoteip6addr} ${host}.${remoteip6domain} ${host}" >> ${tmphostsfile}
                            else
                                echo "${remoteip6addr} ${host}" >> ${tmphostsfile}
                            fi
                        fi
                    done
                fi
            done
        fi
    done
}

analyse_cfnet()
{
    echo "# according settings in /etc/network.d these cfengine peers can be resolved: " >> ${tmphostsfile}
    # loop through peers and calculate the LULA 
    # adresses and write them to the tmphostsfile
    for host in ${networkdir}/*.conf ; do
        . ${host}
        host=${host##*/}
        host=${host%.conf}
        cfaddr6=`ipv6calc -q --in prefix+mac ${GLOBAL_CFNET_PREFIX}:: ${GLOBAL_CFNET_MAC}`
        echo "${cfaddr6} ${host}.${cfdomain}" >> ${tmphostsfile}
    done
}

. /etc/init.d/rc.functions
. /etc/conf.d/rc.conf

thishost=`cat /etc/conf.d/hostname`
cfdomain=`cat /etc/conf.d/domainname`
networkdir="/etc/network.d"
tmphostsfile="/etc/hosts.tmp"
hostsfile="/etc/hosts"

# debug
#logger "$0 $@"

gen_default_hosts ${tmphostsfile}
analyse_cfnet
analyse_staticnets

if ! diff -q ${tmphostsfile} ${hostsfile} >/dev/null ; then 
   mv ${tmphostsfile} ${hostsfile}
else
   rm ${tmphostsfile}
fi


