#!/bin/bash

if [ `id -u` != "0" ] ; then
    echo
    echo "You are not root, please execute me as root."
    echo
    exit 1
fi

if [ ${#} -ne 3 ] ; then
        echo "usage: <image-file.bin.gz> <config.default-file> <device>"
        exit
fi



echo "$1" | grep "bin.gz" > /dev/null

if [ $? -ne 0 ] ; then
        echo "image must be .bin.gz"
        exit
fi

MOUNTPOINT1=/media/part1
MOUNTPOINT2=/media/part2
IMAGE_BUILDER=/home/tom/Documents/cti-mesh-builder

echo "$2" | grep ".default" > /dev/null

if [ $? -ne 0 ] ; then
        echo "configfile must be .default"
        exit
fi



DEVICE=$3

echo "***************************************"
echo "unmounting / creation of mountpoints"
echo "***************************************"
umount ${DEVICE}1
umount ${DEVICE}2

if [ ! -d ${MOUNTPOINT1} ] ; then
	mkdir ${MOUNTPOINT1}
else
	umount ${MOUNTPOINT1}
fi

if [ ! -d ${MOUNTPOINT2} ] ; then
	mkdir ${MOUNTPOINT2}
else
	umount ${MOUNTPOINT2}
fi

echo "***************************************"
echo "unmounted"
echo "***************************************"
mkfs.ext2 -m 1 ${DEVICE}1  
mkfs.ext2 -m 1 ${DEVICE}2

tune2fs -c 0 ${DEVICE}1   
tune2fs -c 0 ${DEVICE}2    

e2fsck ${DEVICE}1
e2fsck ${DEVICE}2


umount ${MOUNTPOINT1}
umount ${MOUNTPOINT2}

mount ${DEVICE}1 ${MOUNTPOINT1}
mount ${DEVICE}2 ${MOUNTPOINT2}

mkdir ${MOUNTPOINT1}/grub
mkdir ${MOUNTPOINT2}/grub

cp /boot/grub/stage* ${MOUNTPOINT1}/grub/
cp /boot/grub/stage* ${MOUNTPOINT2}/grub/

cp /boot/grub/e2fs_stage1_5 ${MOUNTPOINT1}/grub/
cp /boot/grub/e2fs_stage1_5 ${MOUNTPOINT2}/grub/

echo "***************************************"
echo "copying image"
echo "***************************************"
cp ${IMAGE_BUILDER}/$1 ${MOUNTPOINT1}

echo "***************************************"
echo "copied image"
echo "***************************************"

sudo cat > ${MOUNTPOINT1}/grub/menu.lst << EOF

# /grub/menu.lst
# default menu.lst on hda1

#password --md5 [Generate one with 'grub-md5-crypt']
hiddenmenu                                          
default 0
fallback 0
timeout 5 
serial --unit=0 --speed=9600 --word=8 --parity=no --stop=1
terminal serial                                           

title default
root(hd0,0)  
kernel /$1 console=ttyS0,9600n8 reboot=bios panic=5 rdinit=/sbin/init loglevel=0

title update
root (hd0,1)
setup (hd0) 
root (hd0,0)
kernel /alix-image-UPDATE_VERSION.bin.gz console=ttyS0,9600n8 reboot=bios panic=5 rdinit=/sbin/init loglevel=0 UPDATE

EOF


sudo cat > ${MOUNTPOINT2}/grub/menu.lst << EOF

# /grub/menu.lst
# fallback menu.lst on hda2

hiddenmenu                                          
default 0 
fallback 0
timeout 5 
serial --unit=0 --speed=9600 --word=8 --parity=no --stop=1
terminal serial

title default
root (hd0,0)  
setup (hd0)
root (hd0,0)
kernel /alix-image-$1.bin.gz console=ttyS0,9600n8 reboot=bios panic=5 rdinit=/sbin/init loglevel=0

EOF

echo "***************************************"
echo "open grub, configure MBR" 
echo "***************************************"

sudo cat > grubInput.txt << EOF
root (hd1,0)
cat /grub/menu.lst
setup (hd1)
quit
EOF

grub --batch < grubInput.txt

rm grubInput.txt

mkdir ${MOUNTPOINT2}/log
mkdir ${MOUNTPOINT2}/config

echo "copying config"
cp ${IMAGE_BUILDER}/$2 ${MOUNTPOINT2}/config/config.default

umount ${DEVICE}1
umount ${DEVICE}2

umount ${MOUNTPOINT1}
umount ${MOUNTPOINT2}
